@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/bash -x
 
 #    Authors:
 #        Lukas Slebodnik <lslebodn@redhat.com>
@@ -27,6 +27,7 @@ usage(){
     echo -e "\t-c, --clean        Remove directory rpmbuild and exit."
     echo -e "\t-P, --patches      Requires list of patches for SRPM."
     echo -e "\t-o, --output       Moves the created srpm to a specific output directory."
+    echo -e "\t-v, --version      Provide package version to set in spec file."
     echo -e "\t-h, --help         Print this help and exit."
     echo -e "\t-?, --usage"
 
@@ -92,6 +93,11 @@ case $i in
     OUTPUT=("$@")
     break
     ;;
+    -v|--version)
+    shift
+    VERSION=("$@")
+    break
+    ;;
     -h|--help|-\?|--usage)
     usage
     ;;
@@ -134,6 +140,9 @@ fi
 
 PACKAGE_VERSION=$(grep "\[VERSION_NUMBER\]" $VERSION_FILE \
                   | sed -e 's/.*\[//' -e 's/\]).*$//')
+if [ -n "$VERSION" ]; then
+    PACKAGE_VERSION="$VERSION"
+fi
 if [ "x$PACKAGE_VERSION" = x ]; then
     echo "Fatal: Could parse version from file:$VERSION_FILE!"
     exit 1;
@@ -157,10 +166,20 @@ sed -e "s/@PACKAGE_NAME@/$PACKAGE_NAME/" \
     > "$RPMBUILD/SPECS/$PACKAGE_NAME.spec"
 
 NAME="$PACKAGE_NAME-$PACKAGE_VERSION"
+TARBALL="$RPMBUILD/SOURCES/$NAME.tar.gz"
+
 git archive --format=tar --prefix="$NAME"/ \
-            --remote="file://$SRC_DIR" \
-            HEAD \
-            | gzip > "$RPMBUILD/SOURCES/$NAME.tar.gz"
+            HEAD | gzip > "$TARBALL"
+
+# fallback to tar if git archive failed
+# tar may include more files so git archive is preferred
+tar -tzf "$TARBALL" &> /dev/null
+if [ $? -ne 0 ]; then
+    rm -f "$TARBALL"
+    pushd "$SRC_DIR"
+    tar -cvzf "$TARBALL" --transform "s,^,$NAME/," *
+    popd
+fi
 
 cp "$SRC_DIR"/contrib/*.patch "$RPMBUILD/SOURCES" 2>/dev/null
 add_patches "$RPMBUILD/SPECS/$PACKAGE_NAME.spec" \
