@@ -154,7 +154,7 @@ sysdb_gpo_store_gpo(struct sss_domain_info *domain,
             goto done;
         }
 
-        lret = ldb_msg_add_fmt(update_msg, SYSDB_GPO_TIMEOUT_ATTR, "%lu",
+        lret = ldb_msg_add_fmt(update_msg, SYSDB_GPO_TIMEOUT_ATTR, "%"SPRItime,
                                ((cache_timeout) ? (now + cache_timeout) : 0));
         if (lret != LDB_SUCCESS) {
             ret = sysdb_error_to_errno(lret);
@@ -198,7 +198,7 @@ sysdb_gpo_store_gpo(struct sss_domain_info *domain,
             goto done;
         }
 
-        lret = ldb_msg_add_fmt(update_msg, SYSDB_GPO_TIMEOUT_ATTR, "%lu",
+        lret = ldb_msg_add_fmt(update_msg, SYSDB_GPO_TIMEOUT_ATTR, "%"SPRItime,
                                ((cache_timeout) ? (now + cache_timeout) : 0));
         if (lret != LDB_SUCCESS) {
             ret = sysdb_error_to_errno(lret);
@@ -530,20 +530,17 @@ done:
     return ret;
 }
 
-errno_t
-sysdb_gpo_get_gpo_result_setting(TALLOC_CTX *mem_ctx,
-                                 struct sss_domain_info *domain,
-                                 const char *ini_key,
-                                 const char **_ini_value)
+static errno_t
+sysdb_gpo_get_gpo_result_object(TALLOC_CTX *mem_ctx,
+                                struct sss_domain_info *domain,
+                                const char **attrs,
+                                struct ldb_result **_result)
 {
     errno_t ret;
     int lret;
     struct ldb_dn *base_dn;
     TALLOC_CTX *tmp_ctx;
     struct ldb_result *res;
-    const char *ini_value;
-
-    const char *attrs[] = {ini_key, NULL};
 
     tmp_ctx = talloc_new(NULL);
     if (!tmp_ctx) return ENOMEM;
@@ -562,7 +559,7 @@ sysdb_gpo_get_gpo_result_setting(TALLOC_CTX *mem_ctx,
                       LDB_SCOPE_SUBTREE, attrs, SYSDB_GPO_RESULT_FILTER);
     if (lret) {
         DEBUG(SSSDBG_MINOR_FAILURE,
-              "Could not locate GPO Result: [%s]\n",
+              "Could not locate GPO Result object: [%s]\n",
               ldb_strerror(lret));
         ret = sysdb_error_to_errno(lret);
         goto done;
@@ -573,6 +570,43 @@ sysdb_gpo_get_gpo_result_setting(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    *_result = talloc_steal(mem_ctx, res);
+    ret = EOK;
+
+done:
+
+    if (ret == ENOENT) {
+        DEBUG(SSSDBG_TRACE_ALL, "No GPO Result object.\n");
+    } else if (ret) {
+        DEBUG(SSSDBG_OP_FAILURE, "Error: %d (%s)\n", ret, strerror(ret));
+    }
+
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
+
+errno_t
+sysdb_gpo_get_gpo_result_setting(TALLOC_CTX *mem_ctx,
+                                 struct sss_domain_info *domain,
+                                 const char *ini_key,
+                                 const char **_ini_value)
+{
+    errno_t ret;
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_result *res;
+    const char *ini_value;
+
+    const char *attrs[] = {ini_key, NULL};
+
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx) return ENOMEM;
+
+    ret = sysdb_gpo_get_gpo_result_object(tmp_ctx, domain, attrs, &res);
+    if (ret != EOK) {
+        goto done;
+    }
+
     ini_value = ldb_msg_find_attr_as_string(res->msgs[0],
                                             ini_key,
                                             NULL);
@@ -600,61 +634,6 @@ done:
 }
 
 
-errno_t
-sysdb_gpo_get_gpo_result_object(TALLOC_CTX *mem_ctx,
-                                struct sss_domain_info *domain,
-                                struct ldb_result **_result)
-{
-    errno_t ret;
-    int lret;
-    struct ldb_dn *base_dn;
-    TALLOC_CTX *tmp_ctx;
-    struct ldb_result *res;
-
-    tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) return ENOMEM;
-
-    DEBUG(SSSDBG_TRACE_ALL, SYSDB_TMPL_GPO_RESULT_BASE"\n", domain->name);
-
-    base_dn = ldb_dn_new_fmt(tmp_ctx, domain->sysdb->ldb,
-                             SYSDB_TMPL_GPO_RESULT_BASE,
-                             domain->name);
-    if (!base_dn) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    lret = ldb_search(domain->sysdb->ldb, tmp_ctx, &res, base_dn,
-                      LDB_SCOPE_SUBTREE, NULL, SYSDB_GPO_RESULT_FILTER);
-    if (lret) {
-        DEBUG(SSSDBG_MINOR_FAILURE,
-              "Could not locate GPO Result object: [%s]\n",
-              ldb_strerror(lret));
-        ret = sysdb_error_to_errno(lret);
-        goto done;
-    }
-
-    if (res->count == 0) {
-        ret = ENOENT;
-        goto done;
-    }
-
-    *_result = talloc_steal(mem_ctx, res);
-    ret = EOK;
-
-done:
-
-    if (ret == ENOENT) {
-        DEBUG(SSSDBG_TRACE_ALL, "No GPO Result object.\n");
-    } else if (ret) {
-        DEBUG(SSSDBG_OP_FAILURE, "Error: %d (%s)\n", ret, strerror(ret));
-    }
-
-    talloc_free(tmp_ctx);
-    return ret;
-}
-
-
 errno_t sysdb_gpo_delete_gpo_result_object(TALLOC_CTX *mem_ctx,
                                            struct sss_domain_info *domain)
 {
@@ -670,7 +649,7 @@ errno_t sysdb_gpo_delete_gpo_result_object(TALLOC_CTX *mem_ctx,
 
     in_transaction = true;
 
-    ret = sysdb_gpo_get_gpo_result_object(mem_ctx, domain, &res);
+    ret = sysdb_gpo_get_gpo_result_object(mem_ctx, domain, NULL, &res);
     if (ret != EOK && ret != ENOENT) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Could not delete GPO result object: %d\n", ret);
