@@ -2349,7 +2349,7 @@ static errno_t add_object_category(struct ldb_context *ldb,
     }
 
     if (objects == NULL || objects->count == 0) {
-        DEBUG(SSSDBG_TRACE_LIBS, "No objects found, nothing to do.");
+        DEBUG(SSSDBG_TRACE_LIBS, "No objects found, nothing to do.\n");
         ret = EOK;
         goto done;
     }
@@ -2774,44 +2774,26 @@ done:
     return ret;
 }
 
-int sysdb_ts_upgrade_01(struct sysdb_ctx *sysdb, const char **ver)
+int sysdb_upgrade_24(struct sysdb_ctx *sysdb, const char **ver)
 {
     struct upgrade_ctx *ctx;
     errno_t ret;
-    struct ldb_message *msg = NULL;
 
-    ret = commence_upgrade(sysdb, sysdb->ldb, SYSDB_TS_VERSION_0_2, &ctx);
+    ret = commence_upgrade(sysdb, sysdb->ldb, SYSDB_VERSION_0_25, &ctx);
     if (ret) {
         return ret;
     }
 
-    /* Remove @IDXONE from index */
-    talloc_free(msg);
-    msg = ldb_msg_new(ctx);
-    if (msg == NULL) {
-        ret = ENOMEM;
+    ret = sysdb_ldb_mod_index(sysdb, SYSDB_IDX_DELETE, sysdb->ldb, "dataExpireTimestamp");
+    if (ret == ENOENT) { /*nothing to delete */
+        ret = EOK;
+    }
+    if (ret != EOK) {
+        DEBUG(SSSDBG_TRACE_FUNC, "sysdb_ldb_mod_index() failed [%d]: %s\n",
+              ret, sss_strerror(ret));
         goto done;
     }
 
-    msg->dn = ldb_dn_new(msg, sysdb->ldb, "@INDEXLIST");
-    if (msg->dn == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    ret = ldb_msg_add_empty(msg, "@IDXONE", LDB_FLAG_MOD_DELETE, NULL);
-    if (ret != LDB_SUCCESS) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    ret = ldb_modify(sysdb->ldb, msg);
-    if (ret != LDB_SUCCESS) {
-        ret = sysdb_error_to_errno(ret);
-        goto done;
-    }
-
-    /* conversion done, update version number */
     ret = update_version(ctx);
 
 done:
