@@ -22,10 +22,15 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE   /* For strcasestr() in ipa_set_search_bases() */
+#endif
+
 #include <netdb.h>
 #include <ctype.h>
 #include <arpa/inet.h>
 #include <ldb.h>
+#include <string.h>
 
 #include "db/sysdb_selinux.h"
 #include "providers/ipa/ipa_common.h"
@@ -155,9 +160,6 @@ static errno_t ipa_parse_search_base(TALLOC_CTX *mem_ctx,
     case IPA_DESKPROFILE_SEARCH_BASE:
         class_name = "IPA_DESKPROFILE";
         break;
-    case IPA_SUBID_RANGES_SEARCH_BASE:
-        class_name = "IPA_SUBID_RANGES";
-        break;
     default:
         DEBUG(SSSDBG_CONF_SETTINGS,
               "Unknown search base type: [%d]\n", class);
@@ -178,23 +180,10 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
                        struct confdb_ctx *cdb,
                        const char *conf_path,
                        struct data_provider *dp,
+                       bool sdom_add,
                        struct sdap_options **_opts)
 {
-    TALLOC_CTX *tmpctx;
-    char *basedn;
-    char *realm;
-    char *value;
     int ret;
-    int i;
-    bool server_mode;
-    struct ldb_context *ldb;
-
-    ldb = sysdb_ctx_get_ldb(dp->be_ctx->domain->sysdb);
-
-    tmpctx = talloc_new(ipa_opts);
-    if (!tmpctx) {
-        return ENOMEM;
-    }
 
     ipa_opts->id = talloc_zero(ipa_opts, struct sdap_options);
     if (!ipa_opts->id) {
@@ -203,11 +192,13 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
     }
     ipa_opts->id->dp = dp;
 
-    ret = sdap_domain_add(ipa_opts->id,
-                          ipa_opts->id_ctx->sdap_id_ctx->be->domain,
-                          NULL);
-    if (ret != EOK) {
-        goto done;
+    if (sdom_add) {
+        ret = sdap_domain_add(ipa_opts->id,
+                              ipa_opts->id_ctx->sdap_id_ctx->be->domain,
+                              NULL);
+        if (ret != EOK) {
+            goto done;
+        }
     }
 
     /* get sdap options */
@@ -228,38 +219,33 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
         goto done;
     }
 
-    ret = domain_to_basedn(tmpctx,
-                           dp_opt_get_string(ipa_opts->basic, IPA_KRB5_REALM),
-                           &basedn);
+
+    ret = EOK;
+    *_opts = ipa_opts->id;
+
+done:
     if (ret != EOK) {
-        goto done;
+        talloc_zfree(ipa_opts->id);
     }
+    return ret;
+}
 
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic, SDAP_SEARCH_BASE)) {
-        /* FIXME: get values by querying IPA */
-        /* set search base */
-        value = talloc_asprintf(tmpctx, "cn=accounts,%s", basedn);
-        if (!value) {
-            ret = ENOMEM;
-            goto done;
-        }
-        ret = dp_opt_set_string(ipa_opts->id->basic,
-                                SDAP_SEARCH_BASE, value);
-        if (ret != EOK) {
-            goto done;
-        }
+errno_t
+ipa_set_sdap_options(struct ipa_options *ipa_opts,
+                     struct sdap_options *sdap_opts)
+{
+    TALLOC_CTX *tmpctx;
+    errno_t ret;
+    char *realm;
+    char *value;
 
-        DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->id->basic[SDAP_SEARCH_BASE].opt_name,
-                  dp_opt_get_string(ipa_opts->id->basic, SDAP_SEARCH_BASE));
+    tmpctx = talloc_new(ipa_opts);
+    if (!tmpctx) {
+        return ENOMEM;
     }
-    ret = sdap_parse_search_base(ipa_opts->id, ldb, ipa_opts->id->basic,
-                                 SDAP_SEARCH_BASE,
-                                 &ipa_opts->id->sdom->search_bases);
-    if (ret != EOK) goto done;
 
     /* set krb realm */
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic, SDAP_KRB5_REALM)) {
+    if (NULL == dp_opt_get_string(sdap_opts->basic, SDAP_KRB5_REALM)) {
         realm = dp_opt_get_string(ipa_opts->basic, IPA_KRB5_REALM);
         value = talloc_strdup(tmpctx, realm);
         if (value == NULL) {
@@ -267,22 +253,22 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
             ret = ENOMEM;
             goto done;
         }
-        ret = dp_opt_set_string(ipa_opts->id->basic,
+        ret = dp_opt_set_string(sdap_opts->basic,
                                 SDAP_KRB5_REALM, value);
         if (ret != EOK) {
             goto done;
         }
         DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->id->basic[SDAP_KRB5_REALM].opt_name,
-                  dp_opt_get_string(ipa_opts->id->basic, SDAP_KRB5_REALM));
+                  sdap_opts->basic[SDAP_KRB5_REALM].opt_name,
+                  dp_opt_get_string(sdap_opts->basic, SDAP_KRB5_REALM));
     }
 
-    ret = sdap_set_sasl_options(ipa_opts->id,
+    ret = sdap_set_sasl_options(sdap_opts,
                                 dp_opt_get_string(ipa_opts->basic,
                                                   IPA_HOSTNAME),
-                                dp_opt_get_string(ipa_opts->id->basic,
+                                dp_opt_get_string(sdap_opts->basic,
                                                   SDAP_KRB5_REALM),
-                                dp_opt_get_string(ipa_opts->id->basic,
+                                dp_opt_get_string(sdap_opts->basic,
                                                   SDAP_KRB5_KEYTAB));
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Cannot set the SASL-related options\n");
@@ -290,26 +276,85 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
     }
 
     /* fix schema to IPAv1 for now */
-    ipa_opts->id->schema_type = SDAP_SCHEMA_IPA_V1;
+    sdap_opts->schema_type = SDAP_SCHEMA_IPA_V1;
+
+    ret = EOK;
+
+done:
+    talloc_zfree(tmpctx);
+    return ret;
+}
+
+errno_t
+ipa_set_search_bases(struct ipa_options *ipa_opts,
+                     struct confdb_ctx *cdb,
+                     const char *basedn,
+                     const char *conf_path,
+                     struct sdap_domain *sdap_dom)
+{
+    TALLOC_CTX *tmpctx;
+    errno_t ret;
+    struct sdap_domain *sdom;
+    char *value;
+    struct ldb_context *ldb;
+    struct sdap_options *sdap_opts;
+    bool server_mode;
+    int i;
+
+    sdap_opts = ipa_opts->id;
+
+    if (sdap_dom != NULL) {
+        sdom = sdap_dom;
+    } else {
+        sdom = ipa_opts->id->sdom;
+    }
+
+    tmpctx = talloc_new(ipa_opts);
+    if (!tmpctx) {
+        return ENOMEM;
+    }
+
+    ldb = sysdb_ctx_get_ldb(ipa_opts->id->dp->be_ctx->domain->sysdb);
+
+    if (NULL == dp_opt_get_string(sdap_opts->basic, SDAP_SEARCH_BASE)) {
+        value = talloc_asprintf(tmpctx, "cn=accounts,%s", basedn);
+        if (!value) {
+            ret = ENOMEM;
+            goto done;
+        }
+        ret = dp_opt_set_string(sdap_opts->basic,
+                                SDAP_SEARCH_BASE, value);
+        if (ret != EOK) {
+            goto done;
+        }
+
+        DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
+                  sdap_opts->basic[SDAP_SEARCH_BASE].opt_name,
+                  dp_opt_get_string(sdap_opts->basic, SDAP_SEARCH_BASE));
+    }
+    ret = sdap_parse_search_base(sdap_opts, ldb, sdap_opts->basic,
+                                 SDAP_SEARCH_BASE,
+                                 &sdom->search_bases);
+    if (ret != EOK) goto done;
 
     /* set user/group search bases if they are not specified */
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic,
+    if (NULL == dp_opt_get_string(sdap_opts->basic,
                                   SDAP_USER_SEARCH_BASE)) {
-        ret = dp_opt_set_string(ipa_opts->id->basic, SDAP_USER_SEARCH_BASE,
-                                dp_opt_get_string(ipa_opts->id->basic,
+        ret = dp_opt_set_string(sdap_opts->basic, SDAP_USER_SEARCH_BASE,
+                                dp_opt_get_string(sdap_opts->basic,
                                                   SDAP_SEARCH_BASE));
         if (ret != EOK) {
             goto done;
         }
 
         DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->id->basic[SDAP_USER_SEARCH_BASE].opt_name,
-                  dp_opt_get_string(ipa_opts->id->basic,
+                  sdap_opts->basic[SDAP_USER_SEARCH_BASE].opt_name,
+                  dp_opt_get_string(sdap_opts->basic,
                                     SDAP_USER_SEARCH_BASE));
     }
-    ret = sdap_parse_search_base(ipa_opts->id, ldb, ipa_opts->id->basic,
+    ret = sdap_parse_search_base(sdap_opts, ldb, sdap_opts->basic,
                                  SDAP_USER_SEARCH_BASE,
-                                 &ipa_opts->id->sdom->user_search_bases);
+                                 &sdom->user_search_bases);
     if (ret != EOK) goto done;
 
     /* In server mode we need to search both cn=accounts,$SUFFIX and
@@ -319,7 +364,7 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
     server_mode = dp_opt_get_bool(ipa_opts->basic, IPA_SERVER_MODE);
     if (server_mode != false) {
         /* bases is not NULL at this point already */
-        struct sdap_search_base **bases = ipa_opts->id->sdom->user_search_bases;
+        struct sdap_search_base **bases = sdom->user_search_bases;
         struct sdap_search_base *new_base = NULL;
 
         for (i = 0; bases[i] != NULL; i++) {
@@ -345,8 +390,8 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
                 goto done;
             }
 
-            bases = talloc_realloc(ipa_opts->id,
-                                   ipa_opts->id->sdom->user_search_bases,
+            bases = talloc_realloc(sdap_opts,
+                                   sdom->user_search_bases,
                                    struct sdap_search_base*,
                                    i + 2);
 
@@ -357,77 +402,77 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
 
             bases[i] = new_base;
             bases[i+1] = NULL;
-            ipa_opts->id->sdom->user_search_bases = bases;
+            sdom->user_search_bases = bases;
 
             DEBUG(SSSDBG_TRACE_FUNC,
                   "Option %s expanded to cover cn=trusts base\n",
-                  ipa_opts->id->basic[SDAP_USER_SEARCH_BASE].opt_name);
+                  sdap_opts->basic[SDAP_USER_SEARCH_BASE].opt_name);
         }
     }
 
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic,
+    if (NULL == dp_opt_get_string(sdap_opts->basic,
                                   SDAP_GROUP_SEARCH_BASE)) {
-        ret = dp_opt_set_string(ipa_opts->id->basic, SDAP_GROUP_SEARCH_BASE,
-                                dp_opt_get_string(ipa_opts->id->basic,
+        ret = dp_opt_set_string(sdap_opts->basic, SDAP_GROUP_SEARCH_BASE,
+                                dp_opt_get_string(sdap_opts->basic,
                                                   SDAP_SEARCH_BASE));
         if (ret != EOK) {
             goto done;
         }
 
         DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->id->basic[SDAP_GROUP_SEARCH_BASE].opt_name,
-                  dp_opt_get_string(ipa_opts->id->basic,
+                  sdap_opts->basic[SDAP_GROUP_SEARCH_BASE].opt_name,
+                  dp_opt_get_string(sdap_opts->basic,
                                     SDAP_GROUP_SEARCH_BASE));
     }
-    ret = sdap_parse_search_base(ipa_opts->id, ldb, ipa_opts->id->basic,
+    ret = sdap_parse_search_base(sdap_opts, ldb, sdap_opts->basic,
                                  SDAP_GROUP_SEARCH_BASE,
-                                 &ipa_opts->id->sdom->group_search_bases);
+                                 &sdom->group_search_bases);
     if (ret != EOK) goto done;
 
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic,
+    if (NULL == dp_opt_get_string(sdap_opts->basic,
                                   SDAP_NETGROUP_SEARCH_BASE)) {
         value = talloc_asprintf(tmpctx, "cn=ng,cn=alt,%s", basedn);
         if (!value) {
             ret = ENOMEM;
             goto done;
         }
-        ret = dp_opt_set_string(ipa_opts->id->basic, SDAP_NETGROUP_SEARCH_BASE,
+        ret = dp_opt_set_string(sdap_opts->basic, SDAP_NETGROUP_SEARCH_BASE,
                                 value);
         if (ret != EOK) {
             goto done;
         }
 
         DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->id->basic[SDAP_NETGROUP_SEARCH_BASE].opt_name,
-                  dp_opt_get_string(ipa_opts->id->basic,
+                  sdap_opts->basic[SDAP_NETGROUP_SEARCH_BASE].opt_name,
+                  dp_opt_get_string(sdap_opts->basic,
                                     SDAP_NETGROUP_SEARCH_BASE));
     }
-    ret = sdap_parse_search_base(ipa_opts->id, ldb, ipa_opts->id->basic,
+    ret = sdap_parse_search_base(sdap_opts, ldb, sdap_opts->basic,
                                  SDAP_NETGROUP_SEARCH_BASE,
-                                 &ipa_opts->id->sdom->netgroup_search_bases);
+                                 &sdom->netgroup_search_bases);
     if (ret != EOK) goto done;
 
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic,
+    if (NULL == dp_opt_get_string(sdap_opts->basic,
                                   SDAP_HOST_SEARCH_BASE)) {
 
         value = dp_opt_get_string(ipa_opts->basic, IPA_HOST_SEARCH_BASE);
         if (!value) {
-            value = dp_opt_get_string(ipa_opts->id->basic, SDAP_SEARCH_BASE);
+            value = dp_opt_get_string(sdap_opts->basic, SDAP_SEARCH_BASE);
         }
 
-        ret = dp_opt_set_string(ipa_opts->id->basic, SDAP_HOST_SEARCH_BASE,
+        ret = dp_opt_set_string(sdap_opts->basic, SDAP_HOST_SEARCH_BASE,
                                 value);
         if (ret != EOK) {
             goto done;
         }
 
         DEBUG(SSSDBG_CONF_SETTINGS, "Option %s set to %s\n",
-              ipa_opts->id->basic[SDAP_HOST_SEARCH_BASE].opt_name,
+              sdap_opts->basic[SDAP_HOST_SEARCH_BASE].opt_name,
               value);
     }
-    ret = sdap_parse_search_base(ipa_opts->id->basic, ldb, ipa_opts->id->basic,
+    ret = sdap_parse_search_base(sdap_opts->basic, ldb, sdap_opts->basic,
                                  SDAP_HOST_SEARCH_BASE,
-                                 &ipa_opts->id->sdom->host_search_bases);
+                                 &sdom->host_search_bases);
     if (ret != EOK) goto done;
 
     if (NULL == dp_opt_get_string(ipa_opts->basic,
@@ -500,41 +545,45 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
     if (ret != EOK) goto done;
 
 #ifdef BUILD_SUBID
-    if (NULL == dp_opt_get_string(ipa_opts->basic,
-                                  IPA_SUBID_RANGES_SEARCH_BASE)) {
-        value = talloc_asprintf(tmpctx, "cn=subids,%s",
-                                ipa_opts->id->sdom->search_bases[0]->basedn);
-        if (!value) {
+    if (NULL == dp_opt_get_string(sdap_opts->basic,
+                                  SDAP_SUBID_RANGES_SEARCH_BASE)) {
+        /* check deprecated option */
+        value = dp_opt_get_string(ipa_opts->basic, IPA_SUBID_RANGES_SEARCH_BASE);
+        if (value == NULL) { /* set a default */
+            value = talloc_asprintf(tmpctx, "cn=subids,%s",
+                                    sdom->search_bases[0]->basedn);
+        }
+        if (value == NULL) {
             ret = ENOMEM;
             goto done;
         }
 
-        ret = dp_opt_set_string(ipa_opts->basic, IPA_SUBID_RANGES_SEARCH_BASE, value);
+        ret = dp_opt_set_string(sdap_opts->basic, SDAP_SUBID_RANGES_SEARCH_BASE, value);
         if (ret != EOK) {
             goto done;
         }
 
         DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->basic[IPA_SUBID_RANGES_SEARCH_BASE].opt_name,
-                  dp_opt_get_string(ipa_opts->basic,
-                                    IPA_SUBID_RANGES_SEARCH_BASE));
+                  sdap_opts->basic[SDAP_SUBID_RANGES_SEARCH_BASE].opt_name,
+                  dp_opt_get_string(sdap_opts->basic,
+                                    SDAP_SUBID_RANGES_SEARCH_BASE));
     }
-    ret = ipa_parse_search_base(ipa_opts->basic, ldb, ipa_opts->basic,
-                                IPA_SUBID_RANGES_SEARCH_BASE,
-                                &ipa_opts->id->sdom->subid_ranges_search_bases);
+    ret = sdap_parse_search_base(sdap_opts->basic, ldb, sdap_opts->basic,
+                                 SDAP_SUBID_RANGES_SEARCH_BASE,
+                                 &sdom->subid_ranges_search_bases);
     if (ret != EOK) goto done;
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_subid_map,
                        SDAP_OPTS_SUBID_RANGE,
-                       &ipa_opts->id->subid_map);
+                       &sdap_opts->subid_map);
     if (ret != EOK) {
         goto done;
     }
 #endif
 
-    value = dp_opt_get_string(ipa_opts->id->basic, SDAP_DEREF);
+    value = dp_opt_get_string(sdap_opts->basic, SDAP_DEREF);
     if (value != NULL) {
         ret = deref_string_to_val(value, &i);
         if (ret != EOK) {
@@ -543,29 +592,29 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
         }
     }
 
-    if (NULL == dp_opt_get_string(ipa_opts->id->basic,
+    if (NULL == dp_opt_get_string(sdap_opts->basic,
                                   SDAP_SERVICE_SEARCH_BASE)) {
         value = talloc_asprintf(tmpctx, "cn=ipservices,%s",
-                                dp_opt_get_string(ipa_opts->id->basic,
+                                dp_opt_get_string(sdap_opts->basic,
                                                   SDAP_SEARCH_BASE));
         if (!value) {
             ret = ENOMEM;
             goto done;
         }
-        ret = dp_opt_set_string(ipa_opts->id->basic,
+        ret = dp_opt_set_string(sdap_opts->basic,
                                 SDAP_SERVICE_SEARCH_BASE, value);
         if (ret != EOK) {
             goto done;
         }
 
         DEBUG(SSSDBG_TRACE_FUNC, "Option %s set to %s\n",
-                  ipa_opts->id->basic[SDAP_SERVICE_SEARCH_BASE].opt_name,
-                  dp_opt_get_string(ipa_opts->id->basic,
+                  sdap_opts->basic[SDAP_SERVICE_SEARCH_BASE].opt_name,
+                  dp_opt_get_string(sdap_opts->basic,
                                     SDAP_SERVICE_SEARCH_BASE));
     }
-    ret = sdap_parse_search_base(ipa_opts->id, ldb, ipa_opts->id->basic,
+    ret = sdap_parse_search_base(sdap_opts, ldb, sdap_opts->basic,
                                  SDAP_SERVICE_SEARCH_BASE,
-                                 &ipa_opts->id->sdom->service_search_bases);
+                                 &sdom->service_search_bases);
     if (ret != EOK) goto done;
 
     if (NULL == dp_opt_get_string(ipa_opts->basic,
@@ -660,61 +709,61 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
                                 &ipa_opts->views_search_bases);
     if (ret != EOK) goto done;
 
-    ret = sdap_get_map(ipa_opts->id, cdb, conf_path,
+    ret = sdap_get_map(sdap_opts, cdb, conf_path,
                        ipa_attr_map,
                        SDAP_AT_GENERAL,
-                       &ipa_opts->id->gen_map);
+                       &sdap_opts->gen_map);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_user_map,
                        SDAP_OPTS_USER,
-                       &ipa_opts->id->user_map);
+                       &sdap_opts->user_map);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_extend_map_with_list(ipa_opts->id, ipa_opts->id,
+    ret = sdap_extend_map_with_list(sdap_opts, sdap_opts,
                                     SDAP_USER_EXTRA_ATTRS,
-                                    ipa_opts->id->user_map,
+                                    sdap_opts->user_map,
                                     SDAP_OPTS_USER,
-                                    &ipa_opts->id->user_map,
-                                    &ipa_opts->id->user_map_cnt);
+                                    &sdap_opts->user_map,
+                                    &sdap_opts->user_map_cnt);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_group_map,
                        SDAP_OPTS_GROUP,
-                       &ipa_opts->id->group_map);
+                       &sdap_opts->group_map);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_netgroup_map,
                        IPA_OPTS_NETGROUP,
-                       &ipa_opts->id->netgroup_map);
+                       &sdap_opts->netgroup_map);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_host_map,
                        SDAP_OPTS_HOST,
-                       &ipa_opts->id->host_map);
+                       &sdap_opts->host_map);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_hostgroup_map,
                        IPA_OPTS_HOSTGROUP,
@@ -723,16 +772,16 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_service_map,
                        SDAP_OPTS_SERVICES,
-                       &ipa_opts->id->service_map);
+                       &sdap_opts->service_map);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_selinux_user_map,
                        IPA_OPTS_SELINUX_USERMAP,
@@ -741,7 +790,7 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_view_map,
                        IPA_OPTS_VIEW,
@@ -750,7 +799,7 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
         goto done;
     }
 
-    ret = sdap_get_map(ipa_opts->id,
+    ret = sdap_get_map(sdap_opts,
                        cdb, conf_path,
                        ipa_override_map,
                        IPA_OPTS_OVERRIDE,
@@ -760,13 +809,9 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
     }
 
     ret = EOK;
-    *_opts = ipa_opts->id;
 
 done:
     talloc_zfree(tmpctx);
-    if (ret != EOK) {
-        talloc_zfree(ipa_opts->id);
-    }
     return ret;
 }
 
@@ -956,6 +1001,7 @@ static void ipa_resolve_callback(void *private_data, struct fo_server *server)
 }
 
 static errno_t _ipa_servers_init(struct be_ctx *ctx,
+                                 const char *fo_service,
                                  struct ipa_service *service,
                                  struct ipa_options *options,
                                  const char *servers,
@@ -1003,14 +1049,15 @@ static errno_t _ipa_servers_init(struct be_ctx *ctx,
             }
 
             ipa_domain = dp_opt_get_string(options->basic, IPA_DOMAIN);
-            ret = be_fo_add_srv_server(ctx, "IPA", "ldap", ipa_domain,
+            ret = be_fo_add_srv_server(ctx, fo_service, "ldap", ipa_domain,
                                        BE_FO_PROTO_TCP, false, NULL);
             if (ret) {
                 DEBUG(SSSDBG_FATAL_FAILURE, "Failed to add server\n");
                 goto done;
             }
 
-            DEBUG(SSSDBG_TRACE_FUNC, "Added service lookup for service IPA\n");
+            DEBUG(SSSDBG_TRACE_FUNC, "Added service lookup for service [%s]\n",
+                                     fo_service);
             continue;
         }
 
@@ -1021,7 +1068,7 @@ static errno_t _ipa_servers_init(struct be_ctx *ctx,
             goto done;
         }
 
-        ret = be_fo_add_server(ctx, "IPA", list[i], 0, NULL, primary);
+        ret = be_fo_add_server(ctx, fo_service, list[i], 0, NULL, primary);
         if (ret && ret != EEXIST) {
             DEBUG(SSSDBG_FATAL_FAILURE, "Failed to add server\n");
             goto done;
@@ -1036,17 +1083,19 @@ done:
 }
 
 static inline errno_t
-ipa_primary_servers_init(struct be_ctx *ctx, struct ipa_service *service,
-                         struct ipa_options *options, const char *servers)
+ipa_primary_servers_init(struct be_ctx *ctx, const char *fo_service,
+                         struct ipa_service *service, struct ipa_options *options,
+                         const char *servers)
 {
-    return _ipa_servers_init(ctx, service, options, servers, true);
+    return _ipa_servers_init(ctx, fo_service, service, options, servers, true);
 }
 
 static inline errno_t
-ipa_backup_servers_init(struct be_ctx *ctx, struct ipa_service *service,
-                        struct ipa_options *options, const char *servers)
+ipa_backup_servers_init(struct be_ctx *ctx, const char *fo_service,
+                        struct ipa_service *service, struct ipa_options *options,
+                        const char *servers)
 {
-    return _ipa_servers_init(ctx, service, options, servers, false);
+    return _ipa_servers_init(ctx, fo_service, service, options, servers, false);
 }
 
 static int ipa_user_data_cmp(void *ud1, void *ud2)
@@ -1057,12 +1106,13 @@ static int ipa_user_data_cmp(void *ud1, void *ud2)
 int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
                      const char *primary_servers,
                      const char *backup_servers,
+                     const char *realm,
+                     const char *ipa_service,
                      struct ipa_options *options,
                      struct ipa_service **_service)
 {
     TALLOC_CTX *tmp_ctx;
     struct ipa_service *service;
-    char *realm;
     int ret;
 
     tmp_ctx = talloc_new(NULL);
@@ -1070,13 +1120,6 @@ int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
         return ENOMEM;
     }
 
-    realm = dp_opt_get_string(options->basic, IPA_KRB5_REALM);
-    if (!realm) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "No Kerberos realm set\n");
-        ret = EINVAL;
-        goto done;
-    }
-
     service = talloc_zero(tmp_ctx, struct ipa_service);
     if (!service) {
         ret = ENOMEM;
@@ -1089,7 +1132,7 @@ int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
     }
 
     service->krb5_service = krb5_service_new(service, ctx,
-                                             "IPA", realm,
+                                             ipa_service, realm,
                                              true,   /* The configured value */
                                              0,      /* will be set later when */
                                              0);     /* the auth provider is set up */
@@ -1099,13 +1142,13 @@ int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
         goto done;
     }
 
-    ret = be_fo_add_service(ctx, "IPA", ipa_user_data_cmp);
+    ret = be_fo_add_service(ctx, ipa_service, ipa_user_data_cmp);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to create failover service!\n");
         goto done;
     }
 
-    service->sdap->name = talloc_strdup(service, "IPA");
+    service->sdap->name = talloc_strdup(service, ipa_service);
     if (!service->sdap->name) {
         ret = ENOMEM;
         goto done;
@@ -1119,19 +1162,19 @@ int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
         primary_servers = BE_SRV_IDENTIFIER;
     }
 
-    ret = ipa_primary_servers_init(ctx, service, options, primary_servers);
+    ret = ipa_primary_servers_init(ctx, ipa_service, service, options, primary_servers);
     if (ret != EOK) {
         goto done;
     }
 
     if (backup_servers) {
-        ret = ipa_backup_servers_init(ctx, service, options, backup_servers);
+        ret = ipa_backup_servers_init(ctx, ipa_service, service, options, backup_servers);
         if (ret != EOK) {
             goto done;
         }
     }
 
-    ret = be_fo_service_add_callback(memctx, ctx, "IPA",
+    ret = be_fo_service_add_callback(memctx, ctx, ipa_service,
                                      ipa_resolve_callback, service);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to add failover callback!\n");
@@ -1368,3 +1411,113 @@ errno_t ipa_get_host_attrs(struct dp_option *ipa_options,
 done:
     return ret;
 }
+
+struct ipa_options *
+ipa_create_trust_options(TALLOC_CTX *mem_ctx,
+                         struct be_ctx *be_ctx,
+                         struct confdb_ctx *cdb,
+                         const char *subdom_conf_path,
+                         struct data_provider *dp,
+                         struct sss_domain_info *subdom,
+                         const char *keytab,
+                         const char *sasl_authid)
+{
+    struct ipa_options *ipa_options = NULL;
+    struct ipa_id_ctx *ipa_id_ctx = NULL;
+    struct sdap_id_ctx *sdap_id_ctx = NULL;
+    const char *ipa_servers;
+    const char *ipa_backup_servers;
+    const char *service_name;
+    errno_t ret;
+
+    DEBUG(SSSDBG_TRACE_FUNC, "Trust is defined to domain '%s'\n",
+          subdom->name);
+
+    ret = ipa_get_options(mem_ctx, cdb, subdom_conf_path, subdom, &ipa_options);
+    if (ret != EOK || ipa_options == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "ipa_get_options failed\n");
+        goto done;
+    } else if (ipa_options == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "ipa_options is NULL\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ipa_servers = dp_opt_get_string(ipa_options->basic, IPA_SERVER);
+    ipa_backup_servers = dp_opt_get_string(ipa_options->basic, IPA_BACKUP_SERVER);
+
+    service_name = talloc_asprintf(ipa_options, "sd_%s", subdom->name);
+    if (service_name == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = ipa_service_init(ipa_options, be_ctx, ipa_servers,
+                           ipa_backup_servers, subdom->realm,
+                           service_name, ipa_options,
+                           &ipa_options->service);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to init IPA service [%d]: %s\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+    /* Set IPA SDAP options */
+    ipa_id_ctx = talloc_zero(mem_ctx, struct ipa_id_ctx);
+    if (ipa_id_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    sdap_id_ctx = sdap_id_ctx_new(mem_ctx, be_ctx, ipa_options->service->sdap);
+    if (sdap_id_ctx == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ipa_id_ctx->ipa_options = ipa_options;
+    ipa_id_ctx->sdap_id_ctx = sdap_id_ctx;
+    ipa_options->id_ctx = ipa_id_ctx;
+
+    ret = ipa_get_id_options(ipa_options,
+                             cdb,
+                             subdom_conf_path,
+                             dp,
+                             false,
+                             &ipa_options->id);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "ipa_get_id_options failed [%d]: %s\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+    if (keytab != NULL) {
+        ret = dp_opt_set_string(ipa_options->id->basic, SDAP_KRB5_KEYTAB,
+                                keytab);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE, "Cannot set trust keytab\n");
+            goto done;
+        }
+    }
+
+    /* Set SDAP_SASL_AUTHID to the trust principal */
+    ret = dp_opt_set_string(ipa_options->id->basic,
+                            SDAP_SASL_AUTHID, sasl_authid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set SASL authid\n");
+        goto done;
+    }
+
+done:
+    if (ret != EOK) {
+        if (ipa_options != NULL) {
+            talloc_free(ipa_options);
+        }
+        if (ipa_id_ctx != NULL) {
+            talloc_free(ipa_options);
+        }
+        return NULL;
+    }
+
+    return ipa_options;
+}
