@@ -61,14 +61,14 @@ enum ipa_basic_opt {
     IPA_HBAC_SUPPORT_SRCHOST,
     IPA_AUTOMOUNT_LOCATION,
     IPA_RANGES_SEARCH_BASE,
-    IPA_ENABLE_DNS_SITES,
     IPA_SERVER_MODE,
     IPA_VIEWS_SEARCH_BASE,
     IPA_KRB5_CONFD_PATH,
     IPA_DESKPROFILE_SEARCH_BASE,
     IPA_DESKPROFILE_REFRESH,
     IPA_DESKPROFILE_REQUEST_INTERVAL,
-    IPA_SUBID_RANGES_SEARCH_BASE,
+    IPA_SUBID_RANGES_SEARCH_BASE, /* only used if ldap_subid_range_search_base is not set */
+    IPA_ACCESS_ORDER,
 
     IPA_OPTS_BASIC /* opts counter */
 };
@@ -133,6 +133,7 @@ enum ipa_override_attrs {
     IPA_AT_OVERRIDE_GROUP_GID_NUMBER,
     IPA_AT_OVERRIDE_USER_SSH_PUBLIC_KEY,
     IPA_AT_OVERRIDE_USER_CERT,
+    IPA_AT_OVERRIDE_OBJECTCLASS,
 
     IPA_OPTS_OVERRIDE
 };
@@ -208,6 +209,8 @@ struct ipa_id_ctx {
     char *view_name;
     /* Only used with server mode */
     struct ipa_server_mode_ctx *server_mode;
+    const char *global_template_homedir;
+    const char *global_template_shell;
 };
 
 struct ipa_options {
@@ -253,6 +256,7 @@ int ipa_get_id_options(struct ipa_options *ipa_opts,
                        struct confdb_ctx *cdb,
                        const char *conf_path,
                        struct data_provider *dp,
+                       bool sdom_add,
                        struct sdap_options **_opts);
 
 int ipa_get_auth_options(struct ipa_options *ipa_opts,
@@ -261,6 +265,7 @@ int ipa_get_auth_options(struct ipa_options *ipa_opts,
                          struct dp_option **_opts);
 
 int ipa_get_autofs_options(struct ipa_options *ipa_opts,
+                           struct ldb_context *ldb,
                            struct confdb_ctx *cdb,
                            const char *conf_path,
                            struct sdap_options **_opts);
@@ -268,6 +273,15 @@ int ipa_get_autofs_options(struct ipa_options *ipa_opts,
 errno_t ipa_get_dyndns_options(struct be_ctx *be_ctx,
                                struct ipa_options *ctx);
 
+errno_t ipa_set_sdap_options(struct ipa_options *ipa_opts,
+                             struct sdap_options *sdap_opts);
+
+errno_t ipa_set_search_bases(struct ipa_options *ipa_opts,
+                             struct confdb_ctx *cdb,
+                             const char *basedn,
+                             const char *conf_path,
+                             struct sdap_domain *sdap_dom);
+
 errno_t ipa_hostid_init(TALLOC_CTX *mem_ctx,
                         struct be_ctx *be_ctx,
                         struct ipa_id_ctx *id_ctx,
@@ -281,6 +295,8 @@ errno_t ipa_autofs_init(TALLOC_CTX *mem_ctx,
 int ipa_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
                      const char *primary_servers,
                      const char *backup_servers,
+                     const char *realm,
+                     const char *ipa_service,
                      struct ipa_options *options,
                      struct ipa_service **_service);
 
@@ -320,4 +336,15 @@ errno_t ipa_get_host_attrs(struct dp_option *ipa_options,
 errno_t ipa_refresh_init(struct be_ctx *be_ctx,
                          struct ipa_id_ctx *id_ctx);
 
+
+struct ipa_options *
+ipa_create_trust_options(TALLOC_CTX *mem_ctx,
+                         struct be_ctx *be_ctx,
+                         struct confdb_ctx *cdb,
+                         const char *subdom_conf_path,
+                         struct data_provider *dp,
+                         struct sss_domain_info *subdom,
+                         const char *keytab,
+                         const char *sasl_authid);
+
 #endif /* _IPA_COMMON_H_ */
