@@ -933,7 +933,7 @@ ipa_get_trust_type(struct ipa_id_ctx *ipa_ctx,
         if (iter->dom == dom) break;
     }
 
-    return iter->type;
+    return (iter != NULL) ? iter->type : IPA_TRUST_UNKNOWN;
 }
 
 static struct ipa_id_ctx *ipa_get_ipa_id_ctx(struct ipa_id_ctx *ipa_ctx,
@@ -1700,13 +1700,23 @@ static errno_t ipa_get_trusted_apply_override_step(struct tevent_req *req)
     int entry_type;
     size_t groups_count = 0;
     struct ldb_message **groups = NULL;
+    bool has_template = false;
     const char *attrs[] = SYSDB_INITGR_ATTRS;
 
-    if (state->override_attrs != NULL) {
+    if (state->obj_dom->template_homedir != NULL ||
+        state->obj_dom->template_shell != NULL ||
+        state->ipa_ctx->global_template_homedir != NULL ||
+        state->ipa_ctx->global_template_shell != NULL) {
+        has_template = true;
+    }
+
+    if (state->override_attrs != NULL || has_template) {
         /* We are in ipa-server-mode, so the view is the default view by
          * definition. */
         ret = sysdb_apply_default_override(state->obj_dom,
                                            state->override_attrs,
+                                           state->ipa_ctx->global_template_homedir,
+                                           state->ipa_ctx->global_template_shell,
                                            state->obj_msg->dn);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, "sysdb_apply_default_override failed.\n");
@@ -1961,7 +1971,7 @@ fail:
 
 static int ipa_srv_acct_lookup_step(struct tevent_req *req)
 {
-    struct tevent_req *subreq;
+    struct tevent_req *subreq = NULL;
     struct ipa_srv_acct_state *state = tevent_req_data(req,
                                             struct ipa_srv_acct_state);
 
@@ -1976,8 +1986,8 @@ static int ipa_srv_acct_lookup_step(struct tevent_req *req)
                                        state->override_attrs,
                                        state->ar, state->type);
     } else {
-        DEBUG(SSSDBG_TRACE_FUNC, "Trust type is unknown\n");
-        subreq = NULL;
+        DEBUG(SSSDBG_OP_FAILURE, "Unable to determine IPA or AD trust type\n");
+        return EINVAL;
     }
 
     if (subreq == NULL) {
