@@ -1224,6 +1224,11 @@ static errno_t sdap_set_search_base(struct sdap_options *opts,
     case SDAP_IPNETWORK_SEARCH_BASE:
         bases = &sdom->ipnetwork_search_bases;
         break;
+#ifdef BUILD_SUBID
+    case SDAP_SUBID_RANGES_SEARCH_BASE:
+        bases = &sdom->subid_ranges_search_bases;
+        break;
+#endif
     default:
         return EINVAL;
     }
@@ -1252,19 +1257,10 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
                                              struct sdap_domain *sdom)
 {
     int ret;
-    char *naming_context = NULL;
 
-    if (!sdom->search_bases
-            || !sdom->user_search_bases
-            || !sdom->group_search_bases
-            || !sdom->netgroup_search_bases
-            || !sdom->host_search_bases
-            || !sdom->sudo_search_bases
-            || !sdom->iphost_search_bases
-            || !sdom->ipnetwork_search_bases
-            || !sdom->autofs_search_bases) {
-        naming_context = get_naming_context(opts->basic, rootdse);
-        if (naming_context == NULL) {
+    if (!sdom->naming_context) {
+        sdom->naming_context = get_naming_context(sdom, rootdse);
+        if (sdom->naming_context == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "get_naming_context failed.\n");
 
             /* This has to be non-fatal, since some servers offer
@@ -1280,7 +1276,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1288,7 +1284,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->user_search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_USER_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1296,7 +1292,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->group_search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_GROUP_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1304,7 +1300,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->netgroup_search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_NETGROUP_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1312,7 +1308,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->host_search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_HOST_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1320,7 +1316,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->sudo_search_bases) {
        ret = sdap_set_search_base(opts, sdom,
                                    SDAP_SUDO_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1328,7 +1324,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->service_search_bases) {
        ret = sdap_set_search_base(opts, sdom,
                                   SDAP_SERVICE_SEARCH_BASE,
-                                  naming_context);
+                                  sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1336,7 +1332,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->autofs_search_bases) {
        ret = sdap_set_search_base(opts, sdom,
                                   SDAP_AUTOFS_SEARCH_BASE,
-                                  naming_context);
+                                  sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1344,7 +1340,7 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->iphost_search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_IPHOST_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
@@ -1352,14 +1348,23 @@ errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
     if (!sdom->ipnetwork_search_bases) {
         ret = sdap_set_search_base(opts, sdom,
                                    SDAP_IPNETWORK_SEARCH_BASE,
-                                   naming_context);
+                                   sdom->naming_context);
         if (ret != EOK) goto done;
     }
 
+#ifdef BUILD_SUBID
+    /* subid ranges */
+    if (!sdom->subid_ranges_search_bases) {
+        ret = sdap_set_search_base(opts, sdom,
+                                   SDAP_SUBID_RANGES_SEARCH_BASE,
+                                   sdom->naming_context);
+        if (ret != EOK) goto done;
+    }
+#endif
+
     ret = EOK;
 
 done:
-    talloc_free(naming_context);
     return ret;
 }
 
