@@ -34,11 +34,6 @@
 /* The return code is 32bits */
 #define KCM_RETCODE_SIZE 4
 
-/* The maximum length of a request or reply as defined by the RPC
- * protocol. This is the same constant size as MIT KRB5 uses
- */
-#define KCM_PACKET_MAX_SIZE 10*1024*1024
-
 /* KCM operation, its raw input and raw output and result */
 struct kcm_op_io {
     struct kcm_op *op;
@@ -280,7 +275,7 @@ static errno_t kcm_output_construct(TALLOC_CTX *mem_ctx,
     SAFEALIGN_SETMEM_UINT32(repbuf->rcbuf, 0, &c);
 
     if (replen > 0) {
-        rep = talloc_zero_array(mem_ctx, uint8_t, replen);
+        rep = talloc_array(mem_ctx, uint8_t, replen);
         if (rep == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE,
                   "Failed to allocate memory for the message\n");
@@ -355,7 +350,7 @@ static void kcm_send_reply(struct kcm_req_ctx *req_ctx)
 
     cctx = req_ctx->cctx;
 
-    ret = kcm_output_construct(cctx, &req_ctx->op_io, &req_ctx->repbuf);
+    ret = kcm_output_construct(req_ctx, &req_ctx->op_io, &req_ctx->repbuf);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Cannot construct the reply buffer, terminating client\n");
@@ -442,7 +437,7 @@ static errno_t kcm_recv_data(TALLOC_CTX *mem_ctx,
         return E2BIG;
     }
 
-    msg = talloc_zero_array(mem_ctx, uint8_t, msglen);
+    msg = talloc_array(mem_ctx, uint8_t, msglen);
     if (msg == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Failed to allocate memory for the message\n");
