@@ -22,7 +22,6 @@
 #include "util/child_common.h"
 #include "util/authtok.h"
 #include "db/sysdb.h"
-#include "db/sysdb_passkey_user_verification.h"
 #include "responder/pam/pamsrv.h"
 
 #include "responder/pam/pamsrv_passkey.h"
@@ -41,7 +40,6 @@ struct pam_passkey_table_data {
 struct pam_passkey_verification_enum_str pam_passkey_verification_enum_str[] = {
     { PAM_PASSKEY_VERIFICATION_ON, "on" },
     { PAM_PASSKEY_VERIFICATION_OFF, "off" },
-    { PAM_PASSKEY_VERIFICATION_OMIT, "unset" },
     { PAM_PASSKEY_VERIFICATION_INVALID, NULL }
 };
 
@@ -390,27 +388,15 @@ static errno_t passkey_local_verification(TALLOC_CTX *mem_ctx,
 {
     TALLOC_CTX *tmp_ctx;
     errno_t ret;
-    const char *verification_from_ldap;
     char *verify_opts = NULL;
     bool debug_libfido2 = false;
-    enum passkey_user_verification verification = PAM_PASSKEY_VERIFICATION_OMIT;
+    enum passkey_user_verification verification = PAM_PASSKEY_VERIFICATION_ON;
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
         return ENOMEM;
     }
 
-    ret = sysdb_domain_get_passkey_user_verification(tmp_ctx, sysdb, domain_name,
-                                                     &verification_from_ldap);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              "Failed to read passkeyUserVerification from sysdb: [%d]: %s\n",
-              ret, sss_strerror(ret));
-        /* This is expected for AD and LDAP */
-        ret = EOK;
-        goto done;
-    }
-
     ret = confdb_get_bool(pctx->pam_ctx->rctx->cdb, CONFDB_PAM_CONF_ENTRY,
                           CONFDB_PAM_PASSKEY_DEBUG_LIBFIDO2, false,
                           &debug_libfido2);
@@ -421,34 +407,24 @@ static errno_t passkey_local_verification(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    /* If require user verification setting is set in LDAP, use it */
-    if (verification_from_ldap != NULL) {
-        if (strcasecmp(verification_from_ldap, "true") == 0) {
-            verification = PAM_PASSKEY_VERIFICATION_ON;
-        } else if (strcasecmp(verification_from_ldap, "false") == 0) {
-            verification = PAM_PASSKEY_VERIFICATION_OFF;
-        }
-        DEBUG(SSSDBG_TRACE_FUNC, "Passkey verification is being enforced from LDAP\n");
-    } else {
-        /* No verification set in LDAP, fallback to local sssd.conf setting */
-        ret = confdb_get_string(pctx->pam_ctx->rctx->cdb, tmp_ctx, CONFDB_MONITOR_CONF_ENTRY,
-                                CONFDB_MONITOR_PASSKEY_VERIFICATION, NULL,
-                                &verify_opts);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE,
-                "Failed to read '"CONFDB_MONITOR_PASSKEY_VERIFICATION"' from confdb: [%d]: %s\n",
-                ret, sss_strerror(ret));
-            goto done;
-        }
-
-
-        ret = read_passkey_conf_verification(tmp_ctx, verify_opts, &verification);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_MINOR_FAILURE, "Unable to parse passkey verificaton options.\n");
-            /* Continue anyway */
-        }
-        DEBUG(SSSDBG_TRACE_FUNC, "Passkey verification is being enforced from local configuration\n");
+    /* Check local sssd.conf setting */
+    ret = confdb_get_string(pctx->pam_ctx->rctx->cdb, tmp_ctx, CONFDB_MONITOR_CONF_ENTRY,
+                            CONFDB_MONITOR_PASSKEY_VERIFICATION, NULL,
+                            &verify_opts);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+            "Failed to read '"CONFDB_MONITOR_PASSKEY_VERIFICATION"' from confdb: [%d]: %s\n",
+            ret, sss_strerror(ret));
+        goto done;
     }
+
+
+    ret = read_passkey_conf_verification(tmp_ctx, verify_opts, &verification);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "Unable to parse passkey verificaton options.\n");
+        /* Continue anyway */
+    }
+    DEBUG(SSSDBG_TRACE_FUNC, "Passkey verification is being enforced from local configuration\n");
     DEBUG(SSSDBG_TRACE_FUNC, "Passkey verification setting [%s]\n",
                              pam_passkey_verification_enum_to_string(verification));
 
@@ -463,6 +439,32 @@ done:
     return ret;
 }
 
+static bool mapping_is_passkey(TALLOC_CTX *tmp_ctx,
+                               const char *mapping_str)
+{
+    int ret;
+    char **mappings;
+
+    if (mapping_str == NULL) {
+        return false;
+    }
+
+    ret = split_on_separator(tmp_ctx, (const char *) mapping_str, ':', true, true,
+                             &mappings, NULL);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Incorrectly formatted passkey data [%d]: %s\n",
+              ret, sss_strerror(ret));
+        return false;
+    }
+
+    if (strcasecmp(mappings[0], "passkey") != 0) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Mapping data found is not passkey related\n");
+        return false;
+    }
+
+    return true;
+}
+
 errno_t process_passkey_data(TALLOC_CTX *mem_ctx,
                              struct ldb_message *user_mesg,
                              const char *domain,
@@ -471,6 +473,7 @@ errno_t process_passkey_data(TALLOC_CTX *mem_ctx,
     struct ldb_message_element *el;
     TALLOC_CTX *tmp_ctx;
     int ret;
+    int num_creds = 0;
     char **mappings;
     const char **kh_mappings;
     const char **public_keys;
@@ -489,22 +492,6 @@ errno_t process_passkey_data(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    /* This attribute may contain other mapping data unrelated to passkey. In that case
-     * let's omit it. For example, AD user altSecurityIdentities may store ssh public key
-     * or smart card mapping data */
-    ret = split_on_separator(tmp_ctx, (const char *) el->values[0].data, ':', true, true,
-                             &mappings, NULL);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Incorrectly formatted passkey data [%d]: %s\n",
-              ret, sss_strerror(ret));
-        ret = ENOENT;
-        goto done;
-    } else if (strcasecmp(mappings[0], "passkey") != 0) {
-        DEBUG(SSSDBG_TRACE_FUNC, "Mapping data found is not passkey related\n");
-        ret = ENOENT;
-        goto done;
-    }
-
     kh_mappings = talloc_zero_array(tmp_ctx, const char *, el->num_values + 1);
     if (kh_mappings == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "talloc_zero_array failed.\n");
@@ -520,6 +507,12 @@ errno_t process_passkey_data(TALLOC_CTX *mem_ctx,
     }
 
     for (int i = 0; i < el->num_values; i++) {
+        /* This attribute may contain other mapping data unrelated to passkey. In that case
+         * let's skip it. For example, AD user altSecurityIdentities may store ssh public key
+         * or smart card mapping data */
+        if ((mapping_is_passkey(tmp_ctx, (const char *)el->values[i].data)) == false) {
+            continue;
+        }
         ret = split_on_separator(tmp_ctx, (const char *) el->values[i].data, ',', true, true,
                                  &mappings, NULL);
         if (ret != EOK) {
@@ -528,19 +521,26 @@ errno_t process_passkey_data(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        kh_mappings[i] = talloc_strdup(kh_mappings, mappings[0] + strlen(PASSKEY_PREFIX));
-        if (kh_mappings[i] == NULL) {
+        kh_mappings[num_creds] = talloc_strdup(kh_mappings, mappings[0] + strlen(PASSKEY_PREFIX));
+        if (kh_mappings[num_creds] == NULL) {
             DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup key handle failed.\n");
             ret = ENOMEM;
             goto done;
         }
 
-        public_keys[i] = talloc_strdup(public_keys, mappings[1]);
-        if (public_keys[i] == NULL) {
+        public_keys[num_creds] = talloc_strdup(public_keys, mappings[1]);
+        if (public_keys[num_creds] == NULL) {
             DEBUG(SSSDBG_OP_FAILURE, "talloc_strdup public key failed.\n");
             ret = ENOMEM;
             goto done;
         }
+
+        num_creds++;
+    }
+
+    if (num_creds == 0) {
+        ret = ENOENT;
+        goto done;
     }
 
     domain_name = talloc_strdup(tmp_ctx, domain);
@@ -553,7 +553,7 @@ errno_t process_passkey_data(TALLOC_CTX *mem_ctx,
     _data->domain = talloc_steal(mem_ctx, domain_name);
     _data->key_handles = talloc_steal(mem_ctx, kh_mappings);
     _data->public_keys = talloc_steal(mem_ctx, public_keys);
-    _data->num_credentials = el->num_values;
+    _data->num_credentials = num_creds;
 
     ret = EOK;
 done:
@@ -598,7 +598,7 @@ void pam_passkey_get_user_done(struct tevent_req *req)
     int timeout;
     struct cache_req_result *result = NULL;
     struct pk_child_user_data *pk_data = NULL;
-    enum passkey_user_verification verification = PAM_PASSKEY_VERIFICATION_OMIT;
+    enum passkey_user_verification verification = PAM_PASSKEY_VERIFICATION_ON;
 
     pctx = tevent_req_callback_data(req, struct passkey_ctx);
 
@@ -641,7 +641,7 @@ void pam_passkey_get_user_done(struct tevent_req *req)
     DEBUG(SSSDBG_TRACE_ALL, "Processing passkey data\n");
     ret = process_passkey_data(pk_data, result->msgs[0], domain_name, pk_data);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE,
+        DEBUG(SSSDBG_TRACE_FUNC,
               "process_passkey_data failed: [%d]: %s\n",
               ret, sss_strerror(ret));
         goto done;
@@ -707,10 +707,11 @@ done:
         pctx->preq->passkey_data_exists = false;
         pam_check_user_search(pctx->preq);
     } else if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "Unexpected passkey error [%d]: %s."
-                                 " Skipping passkey auth\n",
+        DEBUG(SSSDBG_OP_FAILURE, "Unexpected passkey error [%d]: %s.\n",
                                  ret, sss_strerror(ret));
-        pam_check_user_search(pctx->preq);
+        pctx->preq->passkey_data_exists = false;
+        pctx->preq->pd->pam_status = PAM_SYSTEM_ERR;
+        pam_reply(pctx->preq);
     }
 
     return;
@@ -729,6 +730,7 @@ struct pam_passkey_auth_send_state {
     int timeout;
     int child_status;
     bool kerberos_pa;
+    enum passkey_user_verification user_verification;
 };
 
 static errno_t passkey_child_exec(struct tevent_req *req);
@@ -937,6 +939,7 @@ pam_passkey_auth_send(TALLOC_CTX *mem_ctx,
 
     state->timeout = timeout;
     state->kerberos_pa = kerberos_pa;
+    state->user_verification = verification;
     state->logfile = PASSKEY_CHILD_LOG_FILE;
     state->io = talloc(state, struct child_io_fds);
     if (state->io == NULL) {
@@ -966,7 +969,6 @@ pam_passkey_auth_send(TALLOC_CTX *mem_ctx,
             DEBUG(SSSDBG_TRACE_FUNC, "Calling child with user-verification false\n");
             break;
         default:
-            DEBUG(SSSDBG_TRACE_FUNC, "Calling child with user-verification unset\n");
             break;
     }
 
@@ -1103,8 +1105,9 @@ static errno_t passkey_child_exec(struct tevent_req *req)
             goto done;
         }
 
-        /* PIN is needed */
-        if (sss_authtok_get_type(state->pd->authtok) != SSS_AUTHTOK_TYPE_EMPTY) {
+        /* PIN is needed only when user verification is required */
+        if (sss_authtok_get_type(state->pd->authtok) != SSS_AUTHTOK_TYPE_EMPTY
+            && state->user_verification != PAM_PASSKEY_VERIFICATION_OFF) {
             ret = get_passkey_child_write_buffer(state, state->pd, &write_buf,
                                      &write_buf_len);
             if (ret != EOK) {
@@ -1115,16 +1118,14 @@ static errno_t passkey_child_exec(struct tevent_req *req)
             }
         }
 
-        if (write_buf_len != 0) {
-            subreq = write_pipe_send(state, state->ev, write_buf, write_buf_len,
-                                     state->io->write_to_child_fd);
-            if (subreq == NULL) {
-                DEBUG(SSSDBG_OP_FAILURE, "write_pipe_send failed.\n");
-                ret = ERR_PASSKEY_CHILD;
-                goto done;
-            }
-            tevent_req_set_callback(subreq, passkey_child_write_done, req);
+        subreq = write_pipe_send(state, state->ev, write_buf, write_buf_len,
+                                 state->io->write_to_child_fd);
+        if (subreq == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, "write_pipe_send failed.\n");
+            ret = ERR_PASSKEY_CHILD;
+            goto done;
         }
+        tevent_req_set_callback(subreq, passkey_child_write_done, req);
         /* Now either wait for the timeout to fire or the child to finish */
     } else { /* error */
         ret = errno;
@@ -1320,7 +1321,8 @@ done:
 errno_t pam_eval_passkey_response(struct pam_ctx *pctx,
                                   struct pam_data *pd,
                                   struct pam_auth_req *preq,
-                                  bool *_pk_preauth_done)
+                                  bool *_pk_preauth_done,
+                                  bool *_kerberos)
 {
     struct response_data *pk_resp;
     struct pk_child_user_data *pk_data;
@@ -1337,6 +1339,8 @@ errno_t pam_eval_passkey_response(struct pam_ctx *pctx,
     while (pk_resp != NULL) {
         switch (pk_resp->type) {
         case SSS_PAM_PASSKEY_KRB_INFO:
+            *_kerberos = true;
+
             if (!pctx->passkey_auth) {
                 /* Passkey auth is disabled. To avoid passkey prompts appearing,
                  * don't send SSS_PAM_PASSKEY_KRB_INFO to the client and
