@@ -174,7 +174,7 @@ connect_proxy_command(char **args)
 int main(int argc, const char **argv)
 {
     TALLOC_CTX *mem_ctx = NULL;
-    int pc_debug = SSSDBG_FATAL_FAILURE;
+    int pc_debug = SSSDBG_TOOLS_DEFAULT;
     int pc_port = 22;
     const char *pc_domain = NULL;
     const char *pc_host = NULL;
@@ -224,7 +224,7 @@ int main(int argc, const char **argv)
     while ((ret = poptGetNextOpt(pc)) > 0)
         ;
 
-    DEBUG_INIT(pc_debug);
+    DEBUG_CLI_INIT(pc_debug);
 
     if (ret != -1) {
         BAD_POPT_PARAMS(pc, poptStrerror(ret), ret, fini);
